# Copyright (C) 2024 Impre_visible
from flask_login import UserMixin  # type: ignore
from werkzeug.security import check_password_hash, generate_password_hash
from time import time

from chocolate_app import DB


class Users(DB.Model, UserMixin):  # type: ignore
    """
    Users model

    ...

    Attributes

    ----------

    id : int
    name : str
    password : str
    profil_picture : str
    account_type : str
    """

    id = DB.Column(DB.Integer, primary_key=True, autoincrement=True)
    name = DB.Column(DB.String(255), unique=True)
    password = DB.Column(DB.String(255))
    profil_picture = DB.Column(DB.String(255))
    account_type = DB.Column(DB.String(255))

    def __init__(self, name, password, profil_picture, account_type):
        self.name = name
        if password is not None and password != "":
            self.password = generate_password_hash(password)
        else:
            self.password = None
        self.profil_picture = profil_picture
        self.account_type = account_type

    def __repr__(self) -> str:
        return f"<User {self.name}>"

    def verify_password(self, pwd):
        if not self.password:
            return True
        return check_password_hash(self.password, pwd)


class Movies(DB.Model):  # type: ignore
    """
    Movies model

    ...

    Attributes
    ----------

    id : int
    title : str
    real_title : str
    cover : str
    banner : str
    slug : str
    description : str
    note : str
    date : str
    genre : str
    duration : str
    cast : str
    bande_annonce_url : str
    adult : str
    library_name : str
    alternatives_names : str
    file_date : float
    """

    id = DB.Column(DB.Integer, primary_key=True)
    title = DB.Column(DB.String(255), primary_key=True)
    real_title = DB.Column(DB.String(255), primary_key=True)
    cover = DB.Column(DB.String(255))
    banner = DB.Column(DB.String(255))
    slug = DB.Column(DB.String(255))
    description = DB.Column(DB.String(2550))
    note = DB.Column(DB.String(255))
    date = DB.Column(DB.String(255))
    genre = DB.Column(DB.String(255))
    duration = DB.Column(DB.String(255))
    cast = DB.Column(DB.String(255))
    bande_annonce_url = DB.Column(DB.String(255))
    adult = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))
    alternatives_names = DB.Column(DB.Text)
    file_date = DB.Column(DB.Float)

    def __repr__(self) -> str:
        return f"<Movies {self.title}>"


class Series(DB.Model):  # type: ignore
    """
    Series model

    ...

    Attributes
    ----------

    id : int
    name : str
    original_name : str
    genre : str
    duration : str
    description : str
    cast : str
    bande_annonce_url : str
    cover : str
    banner : str
    note : str
    date : str
    serie_modified_time : float
    library_name : str
    adult : str
    """

    id = DB.Column(DB.Integer, primary_key=True)
    name = DB.Column(DB.String(255), primary_key=True)
    original_name = DB.Column(DB.String(255), primary_key=True)
    genre = DB.Column(DB.String(255))
    duration = DB.Column(DB.String(255))
    description = DB.Column(DB.String(2550))
    cast = DB.Column(DB.String(255))
    bande_annonce_url = DB.Column(DB.String(255))
    cover = DB.Column(DB.String(255))
    banner = DB.Column(DB.String(255))
    note = DB.Column(DB.String(255))
    date = DB.Column(DB.String(255))
    serie_modified_time = DB.Column(DB.Float)
    library_name = DB.Column(DB.String(255))
    adult = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Series {self.name}>"


class Seasons(DB.Model):  # type: ignore
    """
    Seasons model

    ...

    Attributes
    ----------

    serie : int
    season_id : int
    season_number : int
    release : str
    episodes_number : str
    season_name : str
    season_description : str
    cover : str
    modified_date : float
    number_of_episode_in_folder : int
    """

    serie = DB.Column(DB.Integer, nullable=False)
    season_id = DB.Column(DB.Integer, primary_key=True)
    season_number = DB.Column(DB.Integer, primary_key=True)
    release = DB.Column(DB.String(255))
    episodes_number = DB.Column(DB.String(255))
    season_name = DB.Column(DB.String(255))
    season_description = DB.Column(DB.Text)
    cover = DB.Column(DB.String(255))
    modified_date = DB.Column(DB.Float)
    number_of_episode_in_folder = DB.Column(DB.Integer)

    def __repr__(self) -> str:
        return f"<Seasons {self.serie} {self.season_id}>"


class Episodes(DB.Model):  # type: ignore
    """
    Episodes model

    ...

    Attributes
    ----------
    season_id : int
    episode_id : int
    episode_name : str
    episode_number : int
    episode_description : str
    episode_cover_path : str
    release_date : str
    slug : str
    """

    season_id = DB.Column(DB.Integer, nullable=False)
    episode_id = DB.Column(DB.Integer, primary_key=True)
    episode_name = DB.Column(DB.String(255), primary_key=True)
    episode_number = DB.Column(DB.Integer, primary_key=True)
    episode_description = DB.Column(DB.Text)
    episode_cover_path = DB.Column(DB.String(255))
    release_date = DB.Column(DB.String(255))
    slug = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Episodes {self.season_id} {self.episode_number}>"


class RecurringContent(DB.Model):  # type: ignore
    """
    RecurringContent model for episodes

    ...

    Attributes
    ----------
    episode_id : int
    type: str
    start_time: str
    end_time: str
    """

    episode_id = DB.Column(DB.Integer, primary_key=True)
    type = DB.Column(DB.String(255), primary_key=True)
    start_time = DB.Column(DB.String(255), primary_key=True)
    end_time = DB.Column(DB.String(255), primary_key=True)

    def __repr__(self) -> str:
        return f"<RecurringContent {self.episode_id} {self.type}>"


class Games(DB.Model):  # type: ignore
    """
    Games model

    ...

    Attributes
    ----------
    console : str
    id : int
    title : str
    real_title : str
    cover : str
    description : str
    note : str
    date : str
    genre : str
    slug : str
    library_name : str
    """

    console = DB.Column(DB.String(255), nullable=False)
    id = DB.Column(DB.Integer, primary_key=True)
    title = DB.Column(DB.String(255), primary_key=True)
    real_title = DB.Column(DB.String(255), primary_key=True)
    cover = DB.Column(DB.String(255))
    description = DB.Column(DB.String(2550))
    note = DB.Column(DB.String(255))
    date = DB.Column(DB.String(255))
    genre = DB.Column(DB.String(255))
    slug = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Games {self.title}>"


class OthersVideos(DB.Model):  # type: ignore
    """
    OthersVideos model

    ...

    Attributes
    ----------
    video_hash : str
    title : str
    slug : str
    banner : str
    duration : str
    library_name : str
    """

    video_hash = DB.Column(DB.String(255), primary_key=True)
    title = DB.Column(DB.String(255), primary_key=True)
    slug = DB.Column(DB.String(255))
    banner = DB.Column(DB.String(255))
    duration = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<OthersVideos {self.title}>"


class Books(DB.Model):  # type: ignore
    """
    Books model

    ...

    Attributes
    ----------
    id : int
    title : str
    slug : str
    book_type : str
    cover : str
    library_name : str
    """

    id = DB.Column(DB.Integer, primary_key=True, autoincrement=True)
    title = DB.Column(DB.String(255))
    slug = DB.Column(DB.String(255))
    book_type = DB.Column(DB.String(255))
    cover = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Books {self.title}>"


class Artists(DB.Model):  # type: ignore
    """
    Artists model

    ...

    Attributes
    ----------
    id : int
    name : str
    cover : str
    library_name : str
    """

    id = DB.Column(DB.Text, primary_key=True)
    name = DB.Column(DB.String(255))
    cover = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Artists {self.name}>"


class Albums(DB.Model):  # type: ignore
    """
    Albums model

    ...

    Attributes
    ----------
    artist_id : int
    id : int
    name : str
    dir_name : str
    cover : str
    tracks : str
    library_name : str
    """

    artist_id = DB.Column(DB.Integer, primary_key=True)
    id = DB.Column(DB.Integer, primary_key=True)
    name = DB.Column(DB.String(255))
    dir_name = DB.Column(DB.String(255))
    cover = DB.Column(DB.String(255))
    tracks = DB.Column(DB.Text)
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Albums {self.name}>"


class Tracks(DB.Model):  # type: ignore
    """
    Tracks model

    ...

    Attributes
    ----------
    artist_id : int
    album_id : int
    id : int
    name : str
    slug : str
    duration : int
    cover: str
    library_name : str
    """

    artist_id = DB.Column(DB.Integer)
    album_id = DB.Column(DB.Integer)
    id = DB.Column(DB.Integer, primary_key=True, autoincrement=True)
    name = DB.Column(DB.String(255))
    slug = DB.Column(DB.String(255))
    duration = DB.Column(DB.Integer)
    cover = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Tracks {self.name}>"


class Playlists(DB.Model):  # type: ignore
    """
    Playlist model

    ...

    Attributes
    ----------
    user_id : int
    id : int
    name : str
    tracks : str
    duration : int
    cover : str
    library_name : str
    """

    user_id = DB.Column(DB.Integer)
    id = DB.Column(DB.Integer, primary_key=True, autoincrement=True)
    name = DB.Column(DB.String(255))
    tracks = DB.Column(DB.Text)
    duration = DB.Column(DB.Integer)
    cover = DB.Column(DB.String(255))
    library_name = DB.Column(DB.String(255))

    def __repr__(self) -> str:
        return f"<Playlist {self.name}>"


class Language(DB.Model):  # type: ignore
    """
    Language model

    ...

    Attributes
    ----------
    language : str
    """

    language = DB.Column(DB.String(255), primary_key=True)

    def __repr__(self) -> str:
        return f"<Language {self.language}>"


class Actors(DB.Model):  # type: ignore
    """
    Actors model

    ...

    Attributes
    ----------
    name : str
    actor_id : int
    actor_image : str
    actor_description : str
    actor_birth_date : str
    actor_birth_place : str
    actor_programs : str
    """

    name = DB.Column(DB.String(255), primary_key=True)
    actor_id = DB.Column(DB.Integer, primary_key=True)
    actor_image = DB.Column(DB.Text)
    actor_description = DB.Column(DB.String(2550))
    actor_birth_date = DB.Column(DB.String(255))
    actor_birth_place = DB.Column(DB.String(255))
    actor_programs = DB.Column(DB.Text)

    def __repr__(self) -> str:
        return f"<Actors {self.name}>"


class Libraries(DB.Model):  # type: ignore
    """
    Libraries model

    ...

    Attributes
    ----------
    lib_name : str
    lib_image : str
    lib_type : str
    lib_folder : str
    available_for : str
    """

    lib_name = DB.Column(DB.Text, primary_key=True)
    lib_image = DB.Column(DB.Text)
    lib_type = DB.Column(DB.Text)
    lib_folder = DB.Column(DB.Text)
    available_for = DB.Column(DB.Text)

    def __repr__(self) -> str:
        return f"<Libraries {self.lib_name}>"


class MusicPlayed(DB.Model):  # type: ignore
    """
    MusicPlayed model

    ...

    Attributes
    ----------
    user_id : int
    music_id : int
    play_count : int
    """

    user_id = DB.Column(DB.Integer, primary_key=True)
    music_id = DB.Column(DB.Integer, primary_key=True)
    play_count = DB.Column(DB.Integer)

    def __repr__(self) -> str:
        return f"<MusicPlayed {self.user_id}>"


class MusicLiked(DB.Model):  # type: ignore
    """
    MusicLiked model

    ...

    Attributes
    ----------
    user_id : int
    music_id : int
    liked : int
    liked_at : int
    """

    user_id = DB.Column(DB.Integer, primary_key=True)
    music_id = DB.Column(DB.Integer, primary_key=True)
    liked = DB.Column(DB.Integer)
    liked_at = DB.Column(DB.Integer, default=int(time()))

    def __repr__(self) -> str:
        return f"<MusicLiked {self.user_id}>"


class LatestEpisodeWatched(DB.Model):  # type: ignore
    """
    LatestEpisodeWatched model

    ...

    Attributes
    ----------
    user_id : int
    serie_id : int
    episode_id : int
    """

    user_id = DB.Column(DB.Integer, primary_key=True)
    serie_id = DB.Column(DB.Integer, primary_key=True)
    episode_id = DB.Column(DB.Integer)

    def __repr__(self) -> str:
        return f"<LatestEpisodeWatched {self.user_id}>"


class InviteCodes(DB.Model):  # type: ignore
    """
    InviteCodes model

    ...

    Attributes
    ----------
    code : str
    """

    code = DB.Column(DB.String(255), primary_key=True)

    def __repr__(self) -> str:
        return f"<InviteCode {self.code}>"


class LibrariesMerge(DB.Model):  # type: ignore
    """
    LibrariesMerge model

    ...

    Attributes
    ----------
    parent_lib : str
    child_lib : str
    """

    parent_lib = DB.Column(DB.String(255), primary_key=True)
    child_lib = DB.Column(DB.String(255), primary_key=True)

    def __repr__(self) -> str:
        return f"<LibrariesMerge {self.parent_lib}>"



import sqlalchemy
from chocolate_app.tables import Libraries, Series, Seasons, Episodes, RecurringContent
from chocolate_app import create_app
from chocolate_app.intro import rcd
from chocolate_app import DB


class IntroDetector:
    def __init__(self) -> None:
        pass

    def start_library_detection(self, library: Libraries) -> None:
        """
        Start the detection for a library
        """
        series = Series.query.filter_by(library_name=library.lib_name).all()
        for serie in series:
            self.start_series_detection(serie)

    def start_series_detection(self, serie: Series) -> None:
        """
        Start the detection for a series
        """
        seasons = Seasons.query.filter_by(serie=serie.id).all()
        for season in seasons:
            self.start_season_detection(season)

    def start_season_detection(self, season: Seasons) -> None:
        """
        Start the detection for a season
        """
        episodes = Episodes.query.filter_by(season_id=season.season_id).all()
        episodes_ids = [episode.episode_id for episode in episodes]
        allEpisodesScanned = True
        for episode_id in episodes_ids:
            episode_recurring_contents = RecurringContent.query.filter_by(
                episode_id=episode_id
            ).all()
            if not episode_recurring_contents:
                allEpisodesScanned = False
                break
        if allEpisodesScanned:
            return
        episodes_paths = [episode.slug for episode in episodes]
        data = rcd.detect(episodes_paths)
        for key, value in data.items():
            contents = value
            contents.sort(key=lambda x: x[0])
            intro = (0, 0)
            recap = ()
            outro = ()
            episode = Episodes.query.filter_by(slug=key).first()
            episode_id = episode.episode_id

            if len(contents) > 0:
                intro = contents[0]
            if len(contents) > 2:
                recap = contents[1]
            if len(contents) > 1:
                outro = contents[-1]

            if len(recap) > 2 and recap[0] == outro[0] and recap[1] == outro[1]:
                recap = ()

            if len(intro) > 0:
                self.save_recurring_content(intro, "intro", episode_id)
            if len(recap) > 0:
                self.save_recurring_content(recap, "recap", episode_id)
            if len(outro) > 0:
                self.save_recurring_content(outro, "outro", episode_id)

    def save_recurring_content(
        self, content: tuple, content_type: str, episode_id: int
    ) -> None:
        """
        Save the recurring content to the database
        """
        rec = RecurringContent(
            type=content_type,
            start_time=content[0],
            end_time=content[1],
            episode_id=episode_id,
        )
        try:
            DB.session.add(rec)
            DB.session.commit()
        except Exception:
            DB.session.rollback()

def start():
    detector = IntroDetector()
    app = create_app()
    with app.app_context():
        liraries = Libraries.query.filter_by(lib_type="series").all()
        for library in liraries:
            detector.start_library_detection(library)

if __name__ == "__main__":
    start()


"""A plugin loader"""

import os
import importlib
import yaml

def handle_default(yaml: dict, key: str, default: str = "Unknown") -> str:
    """Handle the default values for the plugin.yaml file"""
    if key not in yaml:
        return default
    else:
        return yaml[key]
    
PLUGIN_FRONTENDS: list[str] = []

def load_plugins(plugins_path: str) -> None:
    """Load plugins from the given path"""
    for folders in os.listdir(plugins_path):
        if os.path.isdir(f"{plugins_path}/{folders}"):
            path = f"{plugins_path}/{folders}"
            yaml_file_path = f"{path}/plugin.yaml"
            if os.path.exists(yaml_file_path):
                with open(yaml_file_path, "r") as yaml_file:
                    plugin = yaml.safe_load(yaml_file)
                    if not plugin:
                        continue
                    plugin_name = handle_default(plugin, "name")
                    plugin_version = handle_default(plugin, "version", "1")
                    plugin_author = handle_default(plugin, "author")
                    plugin_file = handle_default(plugin, "file", "main")

                    plugin_module = os.path.basename(plugin_file).split('.')[0]

                    plugin_file = f"{path}/backend/{plugin_module}.py"
                    from chocolate_app.utils.utils import log

                    plugin_module_name = f"plugin_{plugin_name}"
                    specs = importlib.util.spec_from_file_location(plugin_module_name, plugin_file)
                    if not specs:
                        log("Error", "Plugin Loader", f"Could not load plugin {plugin_name} v{plugin_version} by {plugin_author}\n")
                        continue
                    new_module = importlib.util.module_from_spec(specs)
                    if not specs.loader:
                        log("Error", "Plugin Loader", f"Could not load plugin {plugin_name} v{plugin_version} by {plugin_author}\n")
                        continue

                    from chocolate_app import app
                    with app.app_context():
                        specs.loader.exec_module(new_module)

                    PLUGIN_FRONTENDS.append(f"{path}/frontend")


from flask import Blueprint, jsonify, request

from chocolate_app import config, write_config, tmdb
from chocolate_app.tables import Users, Libraries
from chocolate_app.plugins_loader import events

settings_bp = Blueprint("settings", __name__)


@settings_bp.route("/get_settings", methods=["GET"])
def get_settings():
    all_users = Users.query.all()
    all_libraries = Libraries.query.all()
    users = []
    for user in all_users:
        user = user.__dict__
        del user["_sa_instance_state"]
        user["password"] = "Ratio"
        users.append(user)

    libs = []
    for library in all_libraries:
        library = library.__dict__
        del library["_sa_instance_state"]
        libs.append(library)

    data = {
        "users": users,
        "libraries": libs,
    }

    all_sections = config.sections()
    for section in all_sections:
        section_data = config[section]
        the_data = {}
        for key in section_data:
            the_data[key] = section_data[key]
        data[section] = the_data

    return jsonify(data)


@settings_bp.route("/save_settings", methods=["GET", "POST"])
def save_settings():
    global client_id, client_secret
    body = request.get_json()
    tmdb_api_key = body["tmdbKey"]
    language = body["language"]
    igdb_secret_key = body["igdbSecret"]
    igdb_client_id = body["igdbID"]

    radarr_adress = body["radarrAdress"]
    radarrfolder = body["radarrFolder"]
    radarr_api_key = body["radarrAPI"]
    sonarr_adress = body["sonarrAdress"]
    sonarrfolder = body["sonarrFolder"]
    sonarr_api_key = body["sonarrAPI"]
    readarr_adress = body["readarrAdress"]
    readarrfolder = body["readarrFolder"]
    readarr_api_key = body["readarrAPI"]
    lidarr_adress = body["lidarrAdress"]
    lidarrfolder = body["lidarrFolder"]
    lidarr_api_key = body["lidarrAPI"]

    if radarr_adress != "":
        if radarr_adress.startswith("https://"):
            radarr_adress = radarr_adress.replace("https://", "http://")
        if not radarr_adress.startswith("http://"):
            radarr_adress = f"http://{radarr_adress}"
        config.set("ARRSettings", "radarrurl", radarr_adress)
    if radarrfolder != "":
        radarrfolder = radarrfolder.replace("\\", "/")
        if not radarrfolder.endswith("/"):
            radarrfolder = f"{radarrfolder}/"
        config.set("ARRSettings", "radarrfolder", radarrfolder)
    if radarr_api_key != "":
        config.set("APIKeys", "radarr", radarr_api_key)

    if sonarr_adress != "":
        if sonarr_adress.startswith("https://"):
            sonarr_adress = sonarr_adress.replace("https://", "http://")
        if not sonarr_adress.startswith("http://"):
            sonarr_adress = f"http://{sonarr_adress}"
        config.set("ARRSettings", "sonarrurl", sonarr_adress)
    if sonarrfolder != "":
        sonarrfolder = sonarrfolder.replace("\\", "/")
        if not sonarrfolder.endswith("/"):
            sonarrfolder = f"{sonarrfolder}/"
        config.set("ARRSettings", "sonarrfolder", sonarrfolder)
    if sonarr_api_key != "":
        config.set("APIKeys", "sonarr", sonarr_api_key)

    if readarr_adress != "":
        if readarr_adress.startswith("https://"):
            readarr_adress = readarr_adress.replace("https://", "http://")
        if not readarr_adress.startswith("http://"):
            readarr_adress = f"http://{readarr_adress}"
        config.set("ARRSettings", "readarrurl", readarr_adress)
    if readarrfolder != "":
        readarrfolder = readarrfolder.replace("\\", "/")
        if not readarrfolder.endswith("/"):
            readarrfolder = f"{readarrfolder}/"
        config.set("ARRSettings", "readarrfolder", readarrfolder)
    if readarr_api_key != "":
        config.set("ARRSettings", "readarrurl", readarr_adress)

    if lidarr_adress != "":
        if lidarr_adress.startswith("https://"):
            lidarr_adress = lidarr_adress.replace("https://", "http://")
        if not lidarr_adress.startswith("http://"):
            lidarr_adress = f"http://{lidarr_adress}"
        config.set("ARRSettings", "lidarrurl", lidarr_adress)
    if lidarrfolder != "":
        lidarrfolder = lidarrfolder.replace("\\", "/")
        if not lidarrfolder.endswith("/"):
            lidarrfolder = f"{lidarrfolder}/"
        config.set("ARRSettings", "lidarrfolder", lidarrfolder)
    if lidarr_api_key != "":
        config.set("ARRSettings", "lidarrurl", lidarr_adress)
    if tmdb_api_key != "":
        config.set("APIKeys", "TMDB", tmdb_api_key)
        tmdb.api_key = tmdb_api_key
    if igdb_client_id != "" and igdb_secret_key != "":
        config.set("APIKeys", "igdbid", igdb_client_id)
        config.set("APIKeys", "igdbsecret", igdb_secret_key)
        client_id = igdb_client_id
        client_secret = igdb_secret_key

    if language != "undefined":
        config.set("ChocolateSettings", "language", language.lower())

    try:
        allow_download = body["allowDownloadsCheckbox"]
        if allow_download == "on":
            config.set("ChocolateSettings", "allowdownload", "true")
        else:
            config.set("ChocolateSettings", "allowdownload", "false")
    except Exception:
        config.set("ChocolateSettings", "allowdownload", "false")

    write_config(config)
    
    events.execute_event(events.SETTINGS_UPDATE, config)

    return jsonify({"error": "success"})



import os
import time
import base64
import io

from PIL import Image
from flask import Blueprint, Response, jsonify, request, abort
from werkzeug.security import generate_password_hash

from chocolate_app import DB, get_dir_path, all_auth_tokens, IMAGES_PATH
from chocolate_app.tables import Users, InviteCodes
from chocolate_app.utils.utils import check_authorization, generate_log
from chocolate_app.plugins_loader import events


dir_path = get_dir_path()
users_bp = Blueprint("users", __name__)


@users_bp.route("/get_all_users", methods=["GET"])
def get_all_users():
    all_users = Users.query.filter().all()
    all_users_list = []
    for user in all_users:
        profil_picture = user.profil_picture
        if not os.path.exists(dir_path + profil_picture):
            profil_picture = "/static/img/avatars/defaultUserProfilePic.png"
        user_dict = {
            "name": user.name,
            "profil_picture": profil_picture,
            "account_type": user.account_type,
            "password_empty": True if not user.password else False,
            "id": user.id,
        }
        all_users_list.append(user_dict)
    return jsonify(all_users_list)


@users_bp.route("/login", methods=["POST"])
def login():
    from uuid import uuid4

    auth_token = str(uuid4())
    account_name = request.get_json()["name"]
    account_password = request.get_json()["password"]
    user = Users.query.filter_by(name=account_name).first()
    token = f"Bearer {auth_token}"
    actual_time_in_seconds = int(time.time())
    all_auth_tokens[token] = {"user": account_name, "time": actual_time_in_seconds}
    if user:
        if user.account_type == "Kid":
            generate_log(request, "LOGIN")
            events.execute_event(events.LOGIN, user)
            return jsonify(
                {"id": user.id, "name": user.name, "error": "None", "token": auth_token}
            )
        elif user.verify_password(account_password):
            generate_log(request, "LOGIN")
            events.execute_event(events.LOGIN, user)
            return jsonify(
                {"id": user.id, "name": user.name, "error": "None", "token": auth_token}
            )
        else:
            generate_log(request, "ERROR")
            user = user.__dict__
            user["error"] = "Unauthorized"
            events.execute_event(events.LOGIN, user)
            return jsonify({"error": "Unauthorized"})
    else:
        generate_log(request, "ERROR")
        user = user.__dict__
        user["error"] = "Unauthorized"
        events.execute_event(events.LOGIN, user)
        return jsonify({"error": "Unauthorized"})


@users_bp.route("/create_account", methods=["POST"])
def create_account():
    body = request.get_json()
    account_name = body["username"]
    account_password = body["password"]
    account_type_input = body["type"]

    profil_picture = f"{IMAGES_PATH}/avatars/{account_name}.webp"
    if "profil_picture" not in body:
        profil_picture = "/static/img/avatars/defaultUserProfilePic.png"
    else:
        file_base64 = body["profil_picture"]
        if file_base64.startswith("data:image"):
            file_base64 = file_base64.split(",", 1)[1]

        full_path = profil_picture

        image_data = base64.b64decode(file_base64)

        # Lire l'image à partir des bytes
        image = Image.open(io.BytesIO(image_data))

        # Déterminer le format de l'image
        image_format = image.format.lower()

        # Convertir l'image en format WebP si nécessaire
        if image_format != "webp":
            output_buffer = io.BytesIO()
            image.save(output_buffer, "WEBP")
            output_buffer.seek(0)
            image = Image.open(output_buffer)

        # Enregistrer l'image au format WebP
        image.save(full_path, "WEBP")
        image.close()

    user_exists = Users.query.filter_by(name=account_name).first()
    nb_users = len(Users.query.filter().all())
    if user_exists:
        abort(409)
    account_type_input = account_type_input.lower()
    account_type_input = account_type_input.capitalize()

    if nb_users == 0:
        account_type_input = "Admin"

    new_user = Users(
        name=account_name,
        password=account_password,
        profil_picture=profil_picture,
        account_type=account_type_input,
    )
    DB.session.add(new_user)
    DB.session.commit()
    return jsonify(
        {
            "id": new_user.id,
            "name": new_user.name,
        }
    )


@users_bp.route("/edit_profil", methods=["POST"])
def edit_profil() -> Response:
    authorization = request.headers.get("Authorization")

    if authorization not in all_auth_tokens:
        abort(401, "Unauthorized")

    user = Users.query.filter_by(name=all_auth_tokens[authorization]["user"]).first()

    body = request.get_json()

    user_name = body["name"]
    password = body["password"]

    db_user = Users.query.filter_by(name=user_name).first()

    if not db_user or str(db_user.id) != str(user.id):
        abort(500, "You are not allowed to change the username of this user")

    type = None
    if "type" in body:
        type = body["type"]
    id = body["id"]

    if str(id) != str(user.id) and user.account_type != "Admin":
        abort(401, "Unauthorized")

    username_in_tokens = all_auth_tokens[authorization]["user"]
    user = Users.query.filter_by(name=username_in_tokens).first()
    try:
        f = request.files["image"]
        name, extension = os.path.splitext(f.filename or "")
        profil_picture = f"/static/img/{user_name}{extension}"
        if extension == "":
            profil_picture = "/static/img/avatars/defaultUserProfilePic.png"
    except Exception:
        profil_picture = "/static/img/avatars/defaultUserProfilePic.png"

    user_to_edit = Users.query.filter_by(id=id).first()

    if user_to_edit.name != user_name:
        user_to_edit.name = user_name

    if type and user_to_edit.account_type != type:
        user_to_edit.account_type = type

    if user_to_edit.password != generate_password_hash(password) and len(password) > 0:
        user_to_edit.password = generate_password_hash(password)

    if password == "":
        user_to_edit.password = None
    if (
        user_to_edit.profil_picture != profil_picture
        and "/static/img/avatars/defaultUserProfilePic.png" not in profil_picture
    ):
        f = request.files["profil_picture"]
        f.save(f"{dir_path}{profil_picture}")
        user_to_edit.profil_picture = profil_picture

    DB.session.commit()

    return jsonify(
        {
            "id": user_to_edit.id,
            "name": user_to_edit.name,
        }
    )


@users_bp.route("/delete_account", methods=["POST"])
def delete_account() -> Response:
    authorization = request.headers.get("Authorization")
    check_authorization(request, authorization)
    body = request.get_json()
    if not body:
        abort(400, "Missing body")
    if "id" not in body:
        abort(400, "Missing id")
    id = body["id"]

    user = Users.query.filter_by(id=id).first()
    DB.session.delete(user)
    DB.session.commit()

    events.execute_event(events.USER_DELETE, user)

    return jsonify(
        {
            "id": user.id,
            "name": user.name,
        }
    )


@users_bp.route("/get_profil/<id>")
def get_profil(id: int) -> Response:
    user = Users.query.filter_by(id=id).first()
    profil_picture = user.profil_picture
    if not os.path.exists(profil_picture):
        profil_picture = "/static/img/avatars/defaultUserProfilePic.png"
    user_dict = {
        "name": user.name,
        "profil_picture": profil_picture,
        "account_type": user.account_type,
    }
    return jsonify(user_dict)


@users_bp.route("/is_admin", methods=["GET"])
def is_admin() -> Response:
    authorization = request.headers.get("Authorization")
    check_authorization(request, authorization)
    user = Users.query.filter_by(name=all_auth_tokens[authorization]["user"]).first()
    if user.account_type == "Admin":
        return jsonify(True)
    else:
        return jsonify(False)


@users_bp.route("/invite_exist/<hash>", methods=["GET"])
def invite_exist(hash: str) -> Response:
    can = InviteCodes.query.filter_by(code=hash).first() is not None
    return jsonify(can)


@users_bp.route("/create_invite", methods=["POST"])
def create_invite() -> Response:
    authorization = request.headers.get("Authorization")
    check_authorization(request, authorization)
    user = Users.query.filter_by(name=all_auth_tokens[authorization]["user"]).first()

    if user.account_type != "Admin":
        abort(401, "Unauthorized")

    body = request.get_json()
    code = body["code"]
    new_invite = InviteCodes(code=code)
    DB.session.add(new_invite)
    DB.session.commit()
    return jsonify({"code": code})



# Copyright (C) 2024 Impre_visible
import os
import configparser
import platform
import argparse
import logging
import pathlib
import shutil

from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_cors import CORS
from flask_login import LoginManager  # type: ignore
from flask_migrate import Migrate

from typing import Dict, Any

from tmdbv3api import TMDb  # type: ignore

from chocolate_app.plugins_loader import loader

DB: SQLAlchemy = SQLAlchemy()
MIGRATE: Migrate = Migrate()
LOGIN_MANAGER: LoginManager = LoginManager()
all_auth_tokens: Dict[Any, Any] = {}


class ChocolateException(Exception):
    """Base class for exceptions in Chocolate"""


class UnsupportedSystemDefaultPath(ChocolateException):
    """Raised when the default path for the config file and the database file is not supported by Chocolate"""


class TemplateNotFound(ChocolateException):
    """Raised when a template was not found"""


parser: argparse.ArgumentParser = argparse.ArgumentParser("Chocolate")
parser.add_argument("-c", "--config", help="Path to the config file (a .ini file)")
parser.add_argument("--artefacts", help="Path to the artefacts folder (a folder)")
parser.add_argument("-d", "--db", help="Path to the database file (a .db file)")
parser.add_argument("-i", "--images", help="Path to the images folder (a folder)")
parser.add_argument("-pl", "--plugins", help="Path to the plugins folder (a folder)")
parser.add_argument("-l","--logs", help="Path to the logs file (a .log file)")
parser.add_argument("-ns","--no-scans", help="Disable startup scans", action="store_true")
parser.add_argument("-p", "--port", help="Port to run the server on", type=int)

ARGUMENTS = parser.parse_args()

paths = {
    "Windows": {
        "config": f"{os.getenv('APPDATA')}/Chocolate/config.ini",
        "db": f"{os.getenv('APPDATA')}/Chocolate/database.db",
        "images": f"{os.getenv('APPDATA')}/Chocolate/images",
        "logs": f"{os.getenv('APPDATA')}/Chocolate/server.log",
        "plugins": f"{os.getenv('APPDATA')}/Chocolate/plugins",
        "artefacts": f"{os.getenv('APPDATA')}/Chocolate/artefacts",
        "replace_from": "",
        "replace_to": "",
    },
    "Linux": {
        "config": "/var/chocolate/config.ini",
        "db": "/var/chocolate/database.db",
        "images": "/var/chocolate/images/",
        "plugins": "/var/chocolate/plugins/",
        "logs": "/var/chocolate/server.log",
        "artefacts": "/var/chocolate/artefacts/",
        "replace_from": "/var/chocolate",
        "replace_to": f"{os.getenv('HOME')}/.local/share/chocolate",
    },
    "Darwin": {
        "config": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/config.ini",
        "db": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/database.db",
        "images": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/images/",
        "plugins": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/plugins/",
        "logs": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/server.log",
        "artefacts": f"{os.getenv('HOME')}/Library/Application Support/Chocolate/artefacts/",
        "replace_from": "",
        "replace_to": "",
    },
}

OPERATING_SYSTEM: str = platform.system()

if OPERATING_SYSTEM not in paths.keys():
    raise UnsupportedSystemDefaultPath(
        f"No known default file path for the config / database on your operating system ({OPERATING_SYSTEM}). Please use --config and --database path or create a pull request to add your system to the one supported by Chocolate"
    )

CONFIG_PATH: str = ARGUMENTS.config or paths[OPERATING_SYSTEM]["config"]
CONFIG_PATH = CONFIG_PATH.replace("\\", "/")

DB_PATH: str = ARGUMENTS.db or paths[OPERATING_SYSTEM]["db"]
DB_PATH = DB_PATH.replace("\\", "/")

LOG_PATH: str = ARGUMENTS.logs or paths[OPERATING_SYSTEM]["logs"]
LOG_PATH = LOG_PATH.replace("\\", "/")

IMAGES_PATH: str = ARGUMENTS.images or paths[OPERATING_SYSTEM]["images"]
IMAGES_PATH = IMAGES_PATH.replace("\\", "/")
if IMAGES_PATH.endswith("/"):
    IMAGES_PATH = IMAGES_PATH[:-1]

PLUGINS_PATH: str = ARGUMENTS.plugins or paths[OPERATING_SYSTEM]["plugins"]
PLUGINS_PATH = PLUGINS_PATH.replace("\\", "/")
if PLUGINS_PATH.endswith("/"):
    PLUGINS_PATH = PLUGINS_PATH[:-1]

ARTEFACTS_PATH: str = ARGUMENTS.artefacts or paths[OPERATING_SYSTEM]["artefacts"]
ARTEFACTS_PATH = ARTEFACTS_PATH.replace("\\", "/")
if ARTEFACTS_PATH.endswith("/"):
    ARTEFACTS_PATH = ARTEFACTS_PATH[:-1]

SERVER_PORT: int = ARGUMENTS.port or 8888


def replace_path(path: str) -> str:
    return path.replace(
        paths[OPERATING_SYSTEM]["replace_from"], paths[OPERATING_SYSTEM]["replace_to"]
    )


try:
    if not os.path.isdir(os.path.dirname(CONFIG_PATH)):
        os.mkdir(os.path.dirname(CONFIG_PATH))
except PermissionError:
    CONFIG_PATH = replace_path(CONFIG_PATH)
    DB_PATH = replace_path(DB_PATH)
    LOG_PATH = replace_path(LOG_PATH)
    IMAGES_PATH = replace_path(IMAGES_PATH)

CHUNK_LENGTH: int = 9

if os.getenv("NO_SCANS") == "true":
    ARGUMENTS.no_scans = True


def create_app() -> Flask:
    """
    Create the Flask app

    Returns:
        Flask: The Flask app
    """
    dir_path = pathlib.Path(__package__).parent
    TEMPLATE_FOLDER = f"{dir_path}/templates"

    if not os.path.isdir(IMAGES_PATH):
        os.mkdir(IMAGES_PATH)
    if not os.path.isdir(f"{IMAGES_PATH}/avatars"):
        os.mkdir(f"{IMAGES_PATH}/avatars")

    app = Flask(
        __name__, static_folder=f"{dir_path}/static", template_folder=TEMPLATE_FOLDER
    )

    app.secret_key = os.urandom(24)

    CORS(app, supports_credentials=True, resources={r"/*": {"origins": "*"}})
    app.config["SQLALCHEMY_DATABASE_URI"] = f"sqlite:///{DB_PATH}"
    app.config["MAX_CONTENT_LENGTH"] = 4096 * 4096
    app.config["UPLOAD_FOLDER"] = f"{dir_path}/static/img/"
    app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
    app.config["DIR_PATH"] = dir_path
    app.config["JSON_AS_ASCII"] = False

    from .routes.users import users_bp
    from .routes.settings import settings_bp
    from .routes.libraries import libraries_bp
    from .routes.arr import arr_bp

    app.register_blueprint(users_bp)
    app.register_blueprint(settings_bp)
    app.register_blueprint(libraries_bp)
    app.register_blueprint(arr_bp)

    DB.init_app(app)
    MIGRATE.init_app(app, DB)
    LOGIN_MANAGER.init_app(app)
    LOGIN_MANAGER.login_view = "login"

    return app


def check_dependencies() -> None:
    """
    Check if the dependencies are installed
    """
    if not shutil.which("ffmpeg"):
        logging.warning(
            "ffmpeg is not installed. Chocolate will not be able to play videos."
        )

    if not shutil.which("npm"):
        logging.warning(
            "npm is not installed. Chocolate will not be able to install plugins."
        )

    if not shutil.which("git"):
        logging.warning(
            "git is not installed. Chocolate will not be able to install plugins."
        )

def get_dir_path() -> str:
    """
    Get the directory path of the package

    Returns:
        str: The directory path of the package
    """
    dir_path = os.path.dirname(__file__).replace("\\", "/")

    return dir_path


def create_tmdb() -> TMDb:
    """
    Create the TMDb object

    Returns:
        TMDb: The TMDb object
    """
    tmdb = TMDb()
    api_key_tmdb = config["APIKeys"]["TMDB"]
    if api_key_tmdb == "Empty":
        print(
            "Follow this tutorial to get your TMDB API Key : https://developers.themoviedb.org/3/getting-started/introduction"
        )
    tmdb.api_key = config["APIKeys"]["TMDB"]
    tmdb.language = config["ChocolateSettings"]["language"]

    return tmdb


def get_config() -> configparser.ConfigParser:
    """
    Get the config file

    Returns:
        configparser.ConfigParser: The config file
    """
    if not os.path.exists(CONFIG_PATH):
        logging.warning(
            f"Config file not found at {CONFIG_PATH}. Creating a new one..."
        )

        if not os.path.isdir(os.path.dirname(CONFIG_PATH)):
            os.mkdir(os.path.dirname(CONFIG_PATH))

        with open(f"{get_dir_path()}/empty_config.ini", "r") as empty_config:
            with open(CONFIG_PATH, "w") as config:
                config.write(empty_config.read())

    configParser = configparser.ConfigParser()
    configParser.read(CONFIG_PATH)
    if configParser["ChocolateSettings"]["language"] == "Empty":
        configParser["ChocolateSettings"]["language"] = "EN"

    return configParser


def write_config(config) -> None:
    """
    Write the config file

    Args:
        config (configparser.ConfigParser): The config file
    """
    with open(CONFIG_PATH, "w") as configfile:
        config.write(configfile)


def register_plugins() -> None:
    """
    Register the plugins
    """
    loader.load_plugins(PLUGINS_PATH)


check_dependencies()

config = get_config()
tmdb: TMDb = create_tmdb()
app = create_app()

register_plugins()



from genericpath import isdir
import pathlib
import os
import hashlib
import subprocess
import re
import shutil

from chocolate_app.plugins_loader import loader
from chocolate_app import ChocolateException


def rebuild_frontend():
    """
    Rebuild the frontend

    This function rebuild the frontend, based on the plugins that are installed

    Returns:
        None
    """
    from chocolate_app import config, write_config

    if "PLUGINS" not in config:
        config["PLUGINS"] = {}

    if "plugin_hashs" not in config["PLUGINS"]:
        config["PLUGINS"]["plugin_hashs"] = ""

    plugin_hashs = config["PLUGINS"]["plugin_hashs"]
    new_plugin_hashs = set()

    for plugin_frontend in loader.PLUGIN_FRONTENDS:
        plugin_hash = generate_hash(plugin_frontend)
        new_plugin_hashs.add(plugin_hash)

    plugin_hashs = set(plugin_hashs.split(","))

    if plugin_hashs == new_plugin_hashs:
        return

    new_plugin_hashs = ",".join(new_plugin_hashs)
    config["PLUGINS"]["plugin_hashs"] = new_plugin_hashs
    dir_path = pathlib.Path(__package__).parent.absolute()
    frontend_path = f"{dir_path}/frontend_temp"
    static_path = f"{dir_path}/static"
    template_path = f"{dir_path}/templates"

    if not os.path.exists(static_path):
        os.mkdir(static_path)

    if not os.path.exists(template_path):
        os.mkdir(template_path)

    if os.path.exists(frontend_path):
        shutil.rmtree(frontend_path, ignore_errors=True)
    else:
        os.mkdir(frontend_path)

    git_clone_command = (
        "git clone https://github.com/ChocolateApp/ChocolateReact.git frontend_temp"
    )

    subprocess.run(
        git_clone_command.split(), stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
    )

    frontends = loader.PLUGIN_FRONTENDS
    requirements = []
    for plugin_frontend in frontends:
        plugin_hash = generate_hash(plugin_frontend)
        requirements.extend(
            handle_frontend(plugin_frontend, frontend_path, plugin_hash)
        )

    remove_build_files(dir_path)
    build_frontend(frontend_path, dir_path, requirements)
    delete_frontend_temp(frontend_path)
    write_config(config)


def generate_hash(plugin_frontend: str) -> str:
    """
    Generate a hash for the plugin frontend

    Parameters:
        plugin_frontend (str): The path of the plugin frontend

    Returns:
        str: The hash of the plugin frontend
    """
    hash_md5 = hashlib.md5()
    for folder, subfolders, files in os.walk(plugin_frontend):
        for file in files:
            with open(os.path.join(folder, file), "rb") as f:
                for chunk in iter(lambda: f.read(4096), b""):
                    hash_md5.update(chunk)
                    hash_md5.update(file.encode())
    return hash_md5.hexdigest()


def delete_frontend_temp(frontend_path: str):
    """
    Delete the frontend temp folder

    Parameters:
        frontend_path (str): The path of the frontend

    Returns:
        None
    """
    shutil.rmtree(frontend_path, ignore_errors=True)


def remove_build_files(chocolate_path: str):
    """
    Delete the build files

    Parameters:
        chocolate_path (str): The path of the chocolate app

    Returns:
        None
    """
    template_path = f"{chocolate_path}/templates"
    static_path = f"{chocolate_path}/static"
    folders_to_remove = ["js", "css", "media"]
    for folder in folders_to_remove:
        folder_path = f"{static_path}/{folder}"
        if os.path.exists(folder_path):
            shutil.rmtree(folder_path, ignore_errors=True)

    if os.path.exists(f"{template_path}/index.html"):
        os.remove(f"{template_path}/index.html")


def build_frontend(frontend_path: str, chocolate_path: str, requirements: list):
    """
    Build the frontend

    Parameters:
        frontend_path (str): The path of the frontend
        chocolate_path (str): The path of the chocolate app
        requirements (list): The list of required libraries

    Returns:
        None
    """
    npm_install_command = "npm install"
    npm_build_command = "npm run build"

    os.chdir(frontend_path)

    subprocess.run(
        npm_install_command.split(),
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )

    for requirement in requirements:
        subprocess.run(
            f"npm install {requirement}".split(),
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )

    subprocess.run(npm_build_command.split(), stdout=subprocess.DEVNULL)

    build_folder = f"{frontend_path}/build"

    if not os.path.exists(f"{build_folder}/static"):
        raise ChocolateException(
            "The build folder does not contain a static folder. You should check the error message above"
        )
    # tout les dossiers dans build_folder/static sont copiés dans chocolate_path/static
    for folder in os.listdir(f"{build_folder}/static"):
        if os.path.isdir(f"{build_folder}/static/{folder}"):
            shutil.copytree(
                f"{build_folder}/static/{folder}",
                f"{chocolate_path}/static/{folder}",
                dirs_exist_ok=True,
            )

    shutil.copy(f"{build_folder}/index.html", f"{chocolate_path}/templates/index.html")


def handle_frontend(plugin_frontend: str, frontend_path: str, plugin_hash: str) -> list:
    """
    Handle the frontend of the plugin

    Parameters:
        plugin_frontend (str): The path of the plugin frontend
        frontend_path (str): The path of the frontend
        plugin_hash (str): The hash of the plugin frontend

    Returns:
        requirements (list): The list of required libraries
    """

    frontend_dirs = ["New"]
    requirements = []
    for frontend_dir in frontend_dirs:
        if os.path.exists(f"{plugin_frontend}/{frontend_dir}"):
            if frontend_dir == "New":
                rqrs = handle_new(
                    f"{plugin_frontend}/{frontend_dir}", frontend_path, plugin_hash
                )
                requirements.extend(rqrs)
    return requirements


def handle_new(new_folder: str, frontend_path: str, plugin_hash: str):
    """
    Handle the new folder

    Parameters:
        new_folder (str): The path of the new folder
        frontend_path (str): The path of the frontend
        plugin_hash (str): The hash of the plugin frontend

    Returns:
        list: The list of required libraries
    """
    required_libs: list[str] = []
    for files in os.listdir(new_folder):
        file_path = f"{new_folder}/{files}"
        if os.path.isfile(file_path):
            url_path = file_path.split("/")[-1]
            url_path = url_path.replace("<", ":").replace(">", "")
            url_path = url_path.replace("\\", "/")
            url_path = url_path.split(".")[0]
            param_args = []

            for match in re.finditer(r"<(.*?)>", file_path):
                param_args.append(match.group(1))

            file_name = ""
            login_required = False
            with open(file_path, "r") as f:
                contenu = f.read()

            description_match = re.search(r"/\*(.*?)\*/", contenu, re.DOTALL)
            description = (
                description_match.group(1).strip() if description_match else None
            )

            if not description:
                raise ChocolateException("Description not found in the file")
            description_lines = description.split("\n")
            description_lines = description_lines[1:]
            while description_lines[-1] == "":
                description_lines.pop()
            while description_lines[0] == "":
                description_lines.pop(0)

            for line in description_lines:
                clean_line = line.strip().rstrip().lower()
                if clean_line.startswith("filename"):
                    file_name = line.split(":")[1].strip()
                    file_name = file_name.replace(" ", "")
                if clean_line.startswith("login"):
                    login_required = clean_line.lower().split(":")[1].strip() == "true"
                if clean_line.startswith("requirements"):
                    required_libs.extend(clean_line.split(":")[1].strip().split(","))

            if not file_name:
                raise ChocolateException("Filename not found in the file")

            file_name = f"{file_name.split('.')[0]}_{plugin_hash}.jsx"

            new_file_code = f"{contenu}"
            new_file_code = re.sub(
                r"export default function (.*?)\(",
                f"export default function \\1_{plugin_hash}(",
                new_file_code,
            )

            import_string = f"import {file_name.replace('.jsx', '')} from './Pages/{file_name.replace('.jsx', '')}'"
            route_string = f"          <Route path=\"/{url_path}\" element={{<{file_name.replace('.jsx', '')}/>}}"
            if not login_required:
                route_string = f"{route_string} no_login={{true}}"
            route_string = f"{route_string} />"

            app_file = f"{frontend_path}/src/App.js"
            new_app_file = ""
            with open(app_file, "r") as f:
                lines = f.readlines()
                last_import_index = next(
                    (
                        i
                        for i, line in enumerate(lines)
                        if not line.startswith("import ")
                    ),
                    len(lines),
                )
                lines.insert(last_import_index, import_string + "\n")

                routes_end_index = next(
                    (i for i, line in enumerate(lines) if line.strip() == "</Routes>"),
                    len(lines),
                )

                lines.insert(routes_end_index, route_string + "\n")

                new_app_file = "".join(lines)

            with open(app_file, "w") as f:
                f.write(new_app_file)
                pass

            create_jsx_file(f"{frontend_path}/src/Pages/{file_name}", new_file_code)

    return required_libs

def handle_append(append_folder: str, frontend_path: str, plugin_hash: str):
    """
    Handle the append folder (WIP - Not implemented yet)

    Parameters:
        append_folder (str): The path of the append folder
        frontend_path (str): The path of the frontend
        plugin_hash (str): The hash of the plugin frontend

    Returns:

    """
    required_libs: list[str] = []
    for files in os.listdir(append_folder):
        file_path = f"{append_folder}/{files}"
        if os.path.isfile(file_path):
            with open(file_path, "r") as f:
                contenu = f.read()

            description_match = re.search(r"/\*(.*?)\*/", contenu, re.DOTALL)
            description = (
                description_match.group(1).strip() if description_match else None
            )

            if not description:
                raise ChocolateException("Description not found in the file")
            
            description_lines = description.split("\n")
            description_lines = description_lines[1:]
            while description_lines[-1] == "":
                description_lines.pop()
            while description_lines[0] == "":
                description_lines.pop(0)

            for line in description_lines:
                clean_line = line.strip().rstrip().lower()
                if clean_line.startswith("folder"):
                    folder = line.split(":")[1].strip()
                    folder = folder.replace(" ", "")
                elif clean_line.startswith("file"):
                    file = line.split(":")[1].strip()
                elif clean_line.startswith("parent"):
                    parent = clean_line.split(":")[1].strip()
                elif clean_line.startswith("requirements"):
                    required_libs.extend(clean_line.split(":")[1].strip().split(","))
                    
            if not folder:
                raise ChocolateException("Folder not found in the Append file")

            if not file:
                raise ChocolateException("File not found in the Append file")
            
            if not parent:
                raise ChocolateException("Parent not found in the Append file")
            
            if not parent.startswith(".") and not parent.startswith("#"):
                raise ChocolateException("Parent should start with a dot or a hash (class or id)")
            
            import_start_index = 0
            import_end_index = 0
            functions_start_index = 0
            functions_end_index = 0
            component_start_index = 0
            content = contenu.split("\n")
            line = content[functions_start_index]

            if "/* Imports */" not in content:
                raise ChocolateException("Imports section not found in the file")
            
            if "/* Functions */" not in content:
                raise ChocolateException("Functions section not found in the file")
            
            if "/* Component */" not in content:
                raise ChocolateException("Component section not found in the file")

            while line != "/* Imports */":
                import_start_index += 1
                line = content[import_start_index]

            import_end_index = import_start_index
            line = content[import_end_index]

            while line != "/* Functions */":
                import_end_index += 1
                line = content[import_end_index]

            functions_start_index = import_end_index
            functions_end_index = functions_start_index

            line = content[functions_end_index]
            while line != "/* Component */":
                functions_end_index += 1
                line = content[functions_end_index]

            component_start_index = functions_end_index
            component_end_index = len(content)

            import_string = "\n".join(content[import_start_index + 1: import_end_index])
            functions_string = "\n".join(content[functions_start_index + 1: functions_end_index])
            component_string = "\n".join(content[component_start_index + 1: component_end_index])

            with open(f"{frontend_path}/src/{folder}/{file}", "r") as f:
                contenu = f.read()

            new_file_code = contenu
            new_file_code = import_string + new_file_code

            #only the last return statement is replaced
            new_file_code = re.sub(
                r"return (.*?)\(.*?\);",
                f"return \\1_{plugin_hash}();",
                new_file_code,
                count=1
            )

            # the component is added based on the parent query
            lines = new_file_code.split("\n")

            for i in range(len(lines)):
                if parent.startswith("."):
                    if parent[1:] in lines[i] and "className" in lines[i]:
                        while ">" not in lines[i]:
                            i += 1
                        lines.insert(i + 1, component_string)
                        break
                elif parent.startswith("#"):
                    if parent[1:] in lines[i] and "id" in lines[i]:
                        while ">" not in lines[i]:
                            i += 1
                        lines.insert(i + 1, component_string)
                        break

            new_file_code = "\n".join(lines)

            rewrite_jsx_file(f"{frontend_path}/src/{folder}/{file}", new_file_code)


def create_jsx_file(path: str, content: str) -> None:
    """
    Create a JSX file

    Parameters:
        path (str): The path of the JSX file
        content (str): The content of the JSX file

    Returns:
        None
    """
    with open(path, "w") as file:
        file.write(content)

def rewrite_jsx_file(path: str, content: str) -> None:
    """
    Rewrite a JSX file

    Parameters:
        path (str): The path of the JSX file
        content (str): The content of the JSX file

    Returns:
        None
    """
    os.remove(path)
    create_jsx_file(path, content)